=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2005-12-24 19:55:19 +0000
+++ bzrlib/msgeditor.py	2006-01-05 21:36:37 +0000
@@ -87,7 +87,7 @@
             hasinfo = True
             msgfile = file(msgfilename, "w")
             msgfile.write("\n\n%s\n\n%s" % (ignoreline,
-                                        infotext.encode(bzrlib.user_encoding)))
+                infotext.encode(bzrlib.user_encoding, 'replace')))
             msgfile.close()
         else:
             hasinfo = False
@@ -129,6 +129,7 @@
         # delete the msg file in any case
         try: os.unlink(msgfilename)
         except IOError: pass
+        except OSError: pass    # avoid win32 'Permission denied'
 
 
 def make_commit_message_template(working_tree, specific_files):

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2005-12-14 09:01:40 +0000
+++ bzrlib/tests/test_msgeditor.py	2006-01-05 21:36:37 +0000
@@ -20,15 +20,16 @@
 import sys
 
 from bzrlib.branch import Branch
-from bzrlib.msgeditor import make_commit_message_template
+import bzrlib.msgeditor
 from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.trace import mutter
 
 
 class MsgEditorTest(TestCaseInTempDir):
 
     def make_uncommitted_tree(self):
         """Build a branch with uncommitted unicode named changes in the cwd."""
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         working_tree = b.working_tree()
         filename = u'hell\u00d8'
         try:
@@ -42,9 +43,84 @@
     def test_commit_template(self):
         """Test building a commit message template"""
         working_tree = self.make_uncommitted_tree()
-        template = make_commit_message_template(working_tree, None)
+        template = bzrlib.msgeditor.make_commit_message_template(working_tree,
+                                                                 None)
         self.assertEqualDiff(template,
 u"""\
 added:
   hell\u00d8
 """)
+
+    def setUp(self):
+        super(MsgEditorTest, self).setUp()
+        self._bzr_editor = os.environ.get('BZR_EDITOR', None)
+
+    def tearDown(self):
+        if self._bzr_editor != None:
+            os.environ['BZR_EDITOR'] = self._bzr_editor
+        else:
+            if os.environ.get('BZR_EDITOR', None) != None:
+                del os.environ['BZR_EDITOR']
+        super(MsgEditorTest, self).tearDown()
+
+    def test_get_editor(self):
+        os.environ['BZR_EDITOR'] = 'fed'
+        editors = [i for i in bzrlib.msgeditor._get_editor()]
+        self.assertNotEqual([], editors, 'No editor found')
+        self.assertEqual('fed', editors[0])
+
+    def test_run_editor(self):
+        if sys.platform == "win32":
+            f = file('fed.bat', 'w')
+            f.write('@rem dummy fed')
+            f.close()
+            os.environ['BZR_EDITOR'] = 'fed.bat'
+        else:
+            f = file('fed.sh', 'wb')
+            f.write('#!/bin/sh\n')
+            f.close()
+            os.chmod('fed.sh', 0755)
+            os.environ['BZR_EDITOR'] = './fed.sh'
+
+        self.assertEqual(True, bzrlib.msgeditor._run_editor(''),
+                         'Unable to run dummy fake editor')
+
+    def test_edit_commit_message(self):
+        working_tree = self.make_uncommitted_tree()
+        # make fake editor
+        f = file('fed.py', 'wb')
+        f.write('#!%s\n' % sys.executable)
+        f.write("""\
+import sys
+if len(sys.argv) == 2:
+    fn = sys.argv[1]
+    f = file(fn, 'rb')
+    s = f.read()
+    f.close()
+    f = file(fn, 'wb')
+    f.write('test message from fed\\n')
+    f.write(s)
+    f.close()
+""")
+        f.close()
+        if sys.platform == "win32":
+            # [win32] make batch file and set BZR_EDITOR
+            f = file('fed.bat', 'w')
+            f.write("""\
+@echo off
+%s fed.py %%1
+""" % sys.executable)
+            f.close()
+            os.environ['BZR_EDITOR'] = 'fed.bat'
+        else:
+            # [non-win32] make python script executable and set BZR_EDITOR
+            os.chmod('fed.py', 0755)
+            os.environ['BZR_EDITOR'] = './fed.py'
+
+        mutter('edit_commit_message without infotext')
+        self.assertEqual('test message from fed\n',
+                         bzrlib.msgeditor.edit_commit_message(''))
+
+        mutter('edit_commit_message with unicode infotext')
+        self.assertEqual('test message from fed\n',
+                         bzrlib.msgeditor.edit_commit_message(u'\u1234'))

